import matplotlib.pylab as plt
from prostate_cancer import simulate_prostate_cancer_data
from sklearn.model_selection import train_test_split
import graphviz 
from sklearn.metrics import confusion_matrix
from sklearn import tree

X,y = simulate_prostate_cancer_data()

y = y[:,0]

# split data into training and test datasets
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.5, shuffle = True)

for i in range(len(X_train)):
    if y_train[i]>0:
        plt.plot(X_train[i,0],X_train[i,1],"or")
    else:
        plt.plot(X_train[i,0],X_train[i,1],"ob")

for i in range(len(X_test)):
    if y_test[i]>0:
        plt.plot(X_test[i,0],X_test[i,1],"xr")
    else:
        plt.plot(X_test[i,0],X_test[i,1],"xb")

plt.xlabel("CBC")
plt.ylabel("PSA")
plt.savefig("../figures/tree_train_test.eps")

plt.clf()
for i in range(len(X_train)):
    if y_train[i]>0:
        plt.plot(X_train[i,0],X_train[i,1],"or")
    else:
        plt.plot(X_train[i,0],X_train[i,1],"ob")

plt.xlabel("CBC")
plt.ylabel("PSA")
plt.savefig("../figures/tree_train.eps")

import pandas as pd

for depth in range(1,7):
    clf = tree.DecisionTreeClassifier(max_depth=depth)
    clf.fit(X_train, y_train)
    p_train = clf.predict(X_train)
    p_test = clf.predict(X_test)

    #plot tree   
    dot_data = tree.export_graphviz(clf, out_file=None) 
    graph = graphviz.Source(dot_data) 
    graph.render("../figures/prostate_tree_depth_"+str(depth)) 
    
    # calculate training and testing error
    cm_train = confusion_matrix(y_train, p_train)
    print("\n***depth:", depth)
    cm_train_df = pd.DataFrame(cm_train, columns=["PN","PP"])
    print("Training data:\n", cm_train_df.to_latex(), sep="")
    tn, fp, fn, tp = cm_train.ravel()
    print(f"TPR = {tp}/({tp}+{fn}) = " + str(round(tp/(tp+fn),2)))
    print(f"FPR = {fp}/({fp}+{tn}) = " + str(round(fp/(fp+tn),2)))
    
    cm_test = confusion_matrix(y_test,p_test)
    cm_test_df = pd.DataFrame(cm_test, columns=["PN","PP"])
    print("\nTest data:\n", cm_test_df.to_latex(), sep="")
    tn, fp, fn, tp = cm_test.ravel()
    print(f"TPR = {tp}/({tp}+{fn}) = " + str(round(tp/(tp+fn),2)))
    print(f"FPR = {fp}/({fp}+{tn}) = " + str(round(fp/(fp+tn),2)))
    






















