import pandas as pd

from collections import Counter
from sklearn import preprocessing
from sklearn import model_selection
from sklearn import svm
from sklearn.metrics import mean_squared_error
from sklearn.metrics import roc_curve, auc
from sklearn.metrics import confusion_matrix
import matplotlib.pyplot as plt
from sklearn import tree

# parse Excel '.xls' file
xls = pd.ExcelFile("./titanic3.xls")
# extract first sheet in Excel file
sheet_1 = xls.parse(0)
# count passengers that survived
#counter = Counter(sheet_1["survived"].values)
#print(counter) # prints 'Counter({0: 809, 1: 500})'
#print("survived:",counter[1]) # prints: 'survived: 500'
#print("survival rate:",counter[1]/(counter[1]+counter[0]))
# prints 'survival rate: 0.3819709702062643'
# get the number of passengers with a body tag
# and their survival status
#counter = Counter(sheet_1.loc[sheet_1["body"].notna(),"survived"].values)
#print(counter) # prints: 'Counter({0: 121})''
#print(sheet_1.groupby("pclass")["survived"].mean())
# prints:
# pclass
# 1    0.619195
# 2    0.429603
# 3    0.255289
# Name: survived, dtype: float64
#print(sheet_1.groupby(["pclass","sex"])["survived"].mean())
# prints:
# pclass  sex   
# 1       female    0.965278
#         male      0.340782
# 2       female    0.886792
#         male      0.146199
# 3       female    0.490741
#         male      0.152130
# Name: survived, dtype: float64
#print(sheet_1.count())
# prints:
# pclass       1309
# survived     1309
# name         1309
# sex          1309
# age          1046
# sibsp        1309
# parch        1309
# ticket       1309
# fare         1308
# cabin         295
# embarked     1307
# boat          486
# body          121
# home.dest     745
# dtype: int64
data = sheet_1.drop(["body","cabin","boat","home.dest"], axis=1)
#print(list(data))
# prints: ['pclass', 'survived', 'name', 'sex', 'age',
# 'sibsp', 'parch', 'ticket', 'fare', 'embarked']
data = data.dropna()
#print(data.count())
# prints:
# pclass      1043
# survived    1043
# name        1043
# sex         1043
# age         1043
# sibsp       1043
# parch       1043
# ticket      1043
# fare        1043
# embarked    1043
# dtype: int64
le = preprocessing.LabelEncoder()
data.sex = le.fit_transform(data.sex)
data.embarked = le.fit_transform(data.embarked)
#print(data[:1])
# prints:
#    pclass  survived                           name  sex   age  sibsp  parch  \
# 0       1         1  Allen, Miss. Elisabeth Walton    0  29.0      0      0   
#    ticket      fare  embarked  
# 0   24160  211.3375         2 
data = data.drop(["name", "ticket"], axis=1)
X = data.drop(['survived'], axis=1).values
y = data['survived'].values
X_train, X_test, y_train, y_test = model_selection.train_test_split(X, y, test_size=0.2, shuffle = True)
clf = svm.SVC()
clf.fit(X_train, y_train)
scores = clf.decision_function(X_test)
preds = clf.predict(X_test)
print(scores[:5])
print(preds[:5])
print(len(y_test))
tn, fp, fn, tp = confusion_matrix(y_test, preds).ravel()
print(tn, fp, fn, tp)
clf = tree.DecisionTreeClassifier()
clf.fit(X_train, y_train)
print(sum(clf.feature_importances_))
dt_scores = clf.predict_proba(X_test)[:, 1]
# prints: 86 33 41 49
# prints: [0 0 0 0 0 0 1 1 0 1]
#train_err = mean_squared_error(y_train,pred_train)
#test_err = mean_squared_error(y_test,pred_test)
#print("train_error:",train_err,"\ntest_error:",test_err)
#prints:
# train_error: 0.117505995204 
# test_error: 0.320574162679
fpr, tpr, thresholds = roc_curve(y_test, scores)
print(sorted(thresholds[::6]))
roc_auc = auc(fpr, tpr)
plt.plot(fpr,tpr,"b-",lw=2,label="SVC ROC (area = %0.2f)"%roc_auc)
fpr, tpr, thresholds = roc_curve(y_test, dt_scores)
roc_auc = auc(fpr, tpr)
plt.plot(fpr,tpr,"r-",lw=2,label="DT ROC (area = %0.2f)"%roc_auc)
plt.plot([0,1],[0,1],"k--",lw=2)
plt.xlim([0.0, 1.0])
plt.ylim([0.0, 1.05])
plt.xlabel('False Positive Rate')
plt.ylabel('True Positive Rate')
plt.title('Receiver operating characteristic for SVC')
plt.legend(loc="lower right")
plt.savefig("./roc_curve_2.png")
plt.close()

x_labels, bar_heights=zip(*sorted(zip(list(data.drop(['survived'], axis=1)),clf.feature_importances_),key=lambda x:float(x[1]),reverse=True))
plt.title("Feature importance for learned decision tree",size=12)
N = range(len(x_labels))
plt.bar(N,bar_heights,0.9,color="g",edgecolor="k",lw=1)
plt.xlim([-0.5,len(x_labels)-0.5])
xticks = [i for i in N]
plt.xticks(xticks,x_labels,rotation="vertical",size=12)
plt.xlabel("Feature",size=12)
plt.ylabel("Importance",size=12)
plt.tight_layout()
plt.savefig("./bar_plot.png")
plt.close()